<?php

namespace App\Http\Controllers\Admin;

use App\Association;
use App\Documents;
use App\DocumentTypes;
use App\Http\Controllers\Controller;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;

class DocumentsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index()
    {
        $association_id = config('global.association_id');
        $documents = Documents::join('document_types', 'document_type_id', '=',
            'document_types.id')
            ->select(['documents.id', 'documents.doc_title', 'documents.doc_hyperlink', 'document_types.doc_type',
                'documents.created_at', 'documents.updated_at'])
            ->where('documents.association_id', '=', $association_id)
            ->orderBy('doc_type', 'asc')->paginate(10);

        return view('documents.index', [
            'documents' => $documents,
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $association_id = config('global.association_id');
        $doc_types = DocumentTypes::where('association_id', '=', $association_id)
            ->orderBy('doc_type')->get();
        return view('documents.create', [
            'doc_types' => $doc_types,
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $association_id = config('global.association_id');
        request()->validate([
            'doc_title' => 'required|max:80',
            'document_type_id' => 'required',
            'fileUpload' => 'required|mimes:doc,docx,pdf,txt,htm,html|max:2048',
        ]);
        if ($files = $request->file('fileUpload')) {
            $destinationPath = 'docs/assoc_'. $association_id .'_docs/'; // upload path
            $profileDocument = "assoc_" . $association_id . "_doc_" . date('YmdHis') . "." . $files->getClientOriginalExtension();
            $files->move($destinationPath, $profileDocument);
            $insert['association_id'] = "$association_id";
            $insert['document_type_id'] = "$request->document_type_id";
            $insert['doc_title'] = "$request->doc_title";
            $insert['doc_hyperlink'] = "/" . $destinationPath . "$profileDocument";
            $insert['created_at'] = now();
            $insert['updated_at'] = now();
        }
        $check = Documents::insertGetId($insert);

        return Redirect::to('admin/documents')
            ->with('success', 'Great! Document has been successfully uploaded.');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Documents  $documents
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function show(Documents $documents, $id)
    {
        $association_id = config('global.association_id');
        $document = Documents::join('document_types', 'document_type_id', '=',
            'document_types.id')
            ->select(['documents.id', 'documents.doc_title', 'documents.doc_hyperlink', 'document_types.doc_type',
                'documents.created_at', 'documents.updated_at'])
            ->where('documents.id', '=', $id)->first();

        return view('documents.show', [
            'document' => $document,
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function edit($id)
    {
        $association_id = config('global.association_id');
        $doc_types = DocumentTypes::where('association_id', '=', $association_id)
            ->orderBy('doc_type')->get();

        $data = Documents::where('id', '=', $id)->first();

        return view('documents.edit', [
            'data' => $data,
            'doc_types' => $doc_types,
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Documents  $documents
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, $id)
    {
        request()->validate([
            'doc_title' => 'required|max:80',
            'document_type_id' => 'required',
        ]);
        $update = [
            'association_id' => $request->association_id,
            'document_type_id' => $request->document_type_id,
            'doc_title' => $request->doc_title,
            'doc_hyperlink' => $request->doc_hyperlink,
            'created_at' => $request->created_at,
            'updated_at' => now(),
        ];

        Documents::where('id', $id)->update($update);
        return Redirect::to('admin/documents')->with('success', 'The document title was updated successfully.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Documents  $documents
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $file_to_delete = Documents::where('id', $id)->pluck('doc_hyperlink')->toArray();
        $document_record = Documents::findOrFail($id);
        $status = 'success';
        $message = 'The document was deleted successfully.';

        try {
            $document_record->delete();
            Storage::delete($file_to_delete);
        } catch (QueryException $e)
        {
            $status = 'alert-danger';
            $message = 'This document is being used. It cannot be deleted.';
        }

        return Redirect::to('admin/documents')->with($status, $message);
    }
}
