<?php

namespace App\Http\Controllers\Admin;
use App\Association;
use App\User;
use App\Barn;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if(!\Auth::user()->can('manage-users')) {
            abort(401, "You are not authorized to make this request.");
        }

        $users = User::join('associations','users.association_id', '=', 'associations.id')
            ->select(['users.id', 'acronym', 'fname', 'lname', 'email',])
            ->orderBy('acronym', 'asc')
            ->orderBy('users.lname', 'asc')
            ->orderBy('users.fname', 'asc')->paginate(10);

        return view('users.index', [
            'users' => $users,
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $association_id = config('global.association_id');
        $associations = Association::orderBy('name')->get(['acronym', 'id']);
        $barns = Barn::select(['id', 'barn_name'])
            ->where('association_id', '=', $association_id)
            ->orderBy('barn_name', 'asc')->get();

        return view('users.create', [
            'barns' => $barns,
            'associations' => $associations,
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $association_id = config('global.association_id');
        $request->validate([
            'name' => ['required', 'string', 'max:255', 'unique:users'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8', 'confirmed'],
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'street_address' => 'required | max:100',
            'city' => 'required | max:60',
            'state' => 'required | max:50',
            'state_abrv' => 'required | max:2',
            'zip' => 'required | max:10',
            'phone' => 'required | max:15',
        ]);

        User::create([
            'association_id' => $association_id,
            'name' => $request->get('name'),
            'password' => $request->get('password'),
            'email' => $request->get('email'),
            'fname' => $request->get('fname'),
            'lname' => $request->get('lname'),
            'street_address' => $request->get('street_address'),
            'city' => $request->get('city'),
            'state' => $request->get('state'),
            'state_abrv' => $request->get('state_abrv'),
            'zip' => $request->get('zip'),
            'phone' => $request->get('phone'),
            'barn' => $request->get('barn'),
        ]);

        $request->session()->flash('alert-success', 'User created successfully.');
        return redirect('/admin/users');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        if(!\Auth::user()->can('manage-users')) {
            abort(401, "You are not authorized to make this request.");
        }

        $user = User::join('associations','users.association_id', '=', 'associations.id')
            ->leftJoin('barns', 'users.barn', '=', 'barns.id')
            ->select(['users.id', 'acronym', 'users.name', 'users.fname', 'users.lname', 'users.email',
                'users.street_address', 'users.city', 'users.state', 'users.state_abrv',
                'users.zip', 'users.phone', 'barn_name', 'users.created_at', 'users.updated_at'])
            ->where('users.id', '=', $id)
            ->orderBy('acronym', 'asc')
            ->orderBy('users.lname', 'asc')
            ->orderBy('users.fname', 'asc')->first();
//dd($user);
        return view('users.show', [
            'user' => $user,
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $association_id = config('global.association_id');
        $barns = Barn::select(['id', 'barn_name'])
            ->where('association_id', '=', $association_id)
            ->orderBy('barn_name', 'asc')->get();
        $user = User::findOrFail($id);
        return view('users.edit')->with('user', $user)->with('barns', $barns);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $user = User::findOrFail($id);

        if (empty(\Request::get('password'))) {
            $data = \Request::except('password');
        } else {
            $data = \Request::all();
        }

        $user->update($data);

        $request->session()->flash('alert-success', 'User edited successfully.');
        return redirect('/admin/users');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     * @throws \Exception
     */
    public function destroy(Request $request, $id)
    {
        $user  = User::findOrFail($id);
        $user->delete();

        $request->session()->flash('alert-success', 'User deleted successfully.');
        return redirect('/admin/users');
    }
}
